@@ -7,6 +7,9 @@ import logging
 
 import simplejson
 
+from httpexceptor import (HTTP404, HTTP415, HTTP412, HTTP409,
+        HTTP400, HTTP304)
+
 from tiddlyweb.model.collections import Tiddlers
 from tiddlyweb.model.bag import Bag
 from tiddlyweb.model.policy import PermissionsError
@@ -17,8 +20,6 @@ from tiddlyweb.store import (NoTiddlerError, NoBagError, NoRecipeError,
 from tiddlyweb.serializer import (Serializer, TiddlerFormatError,
         NoSerializationError)
 from tiddlyweb.util import pseudo_binary, renderable
-from tiddlyweb.web.http import (HTTP404, HTTP415, HTTP412, HTTP409,
-        HTTP400, HTTP304)
 from tiddlyweb import control
 from tiddlyweb.web import util as web
 from tiddlyweb.web.sendtiddlers import send_tiddlers
@@ -44,7 +45,8 @@ def get_revisions(environ, start_response):
     Get the list of revisions for this tiddler.
     """
     tiddler = _determine_tiddler(environ,
-            control.determine_bag_from_recipe)
+            control.determine_bag_from_recipe,
+            revisions=True)
     return _send_tiddler_revisions(environ, start_response, tiddler)
 
 
@@ -65,7 +67,8 @@ def post_revisions(environ, start_response):
     That collection is known as a TiddlerChronicle.
     """
     tiddler = _determine_tiddler(environ,
-            control.determine_bag_from_recipe)
+            control.determine_bag_from_recipe,
+            revisions=True)
     return _post_tiddler_revisions(environ, start_response, tiddler)
 
 
@@ -78,6 +81,36 @@ def put(environ, start_response):
     return _put_tiddler(environ, start_response, tiddler)
 
 
+def _base_tiddler_object(environ, tiddler_name, revisions):
+    """
+    Create a tiddler object, without bag or recipe, based on
+    URI values.
+
+    ``revisions`` is true when the request is for a revisions
+    collection. When true, extension handling is not done on
+    the tiddler_name.
+    """
+    if not revisions:
+        try:
+            revision = web.get_route_value(environ, 'revision')
+            revision = web.handle_extension(environ, revision)
+        except KeyError:
+            tiddler_name = web.handle_extension(environ, tiddler_name)
+            revision = None
+    else:
+        revision = None
+
+    tiddler = Tiddler(tiddler_name)
+    if revision:
+        try:
+            revision = int(revision)
+            tiddler.revision = revision
+        except ValueError, exc:
+            raise HTTP404('%s not a revision of %s: %s' %
+                    (revision, tiddler_name, exc))
+    return tiddler
+
+
 def _check_bag_constraint(environ, bag, constraint):
     """
     Check to see if the bag allows the current user
@@ -120,58 +153,25 @@ def _delete_tiddler(environ, start_response, tiddler):
     return []
 
 
-def _determine_tiddler(environ, bag_finder):
+def _determine_tiddler(environ, bag_finder, revisions=False):
     """
     Determine, using URL info, the target tiddler.
     This can be complicated because of the mechanics
     of recipes and bags.
+
+    Set revisions to True when the current request ends in
+    `/revisions` or `/revisions.*`. Doing so ensures that
+    processing of extensions does not impact the name of
+    the tiddler.
     """
     tiddler_name = web.get_route_value(environ, 'tiddler_name')
-    try:
-        revision = web.get_route_value(environ, 'revision')
-        revision = web.handle_extension(environ, revision)
-    except KeyError:
-        tiddler_name = web.handle_extension(environ, tiddler_name)
-        revision = None
-
-    tiddler = Tiddler(tiddler_name)
-    if revision:
-        try:
-            revision = int(revision)
-            tiddler.revision = revision
-        except ValueError, exc:
-            raise HTTP404('%s not a revision of %s: %s' %
-                    (revision, tiddler_name, exc))
+    tiddler = _base_tiddler_object(environ, tiddler_name, revisions)
 
     # We have to deserialize the tiddler here so that
     # PUTs to recipes are aware of values like tags when
     # doing filter checks.
     if environ['REQUEST_METHOD'] == 'PUT':
-        length, content_type = web.content_length_and_type(environ)
-        content = web.read_request_body(environ, length)
-
-        try:
-            try:
-                # XXX HACK! We don't want to decode content unless
-                # the serializer has a as_tiddler. We should be able
-                # to just rely on NoSerializationError, but we need
-                # to call the method to do that, and to call the method we
-                # need to decode the string...
-                serialize_type = web.get_serialize_type(environ)[0]
-                serializer = Serializer(serialize_type, environ)
-                serializer.object = tiddler
-                try:
-                    serializer.from_string(content.decode('utf-8'))
-                except TiddlerFormatError, exc:
-                    raise HTTP400('unable to put tiddler: %s' % exc)
-            except NoSerializationError:
-                tiddler.type = content_type
-                if pseudo_binary(tiddler.type):
-                    tiddler.text = content.decode('utf-8')
-                else:
-                    tiddler.text = content
-        except UnicodeDecodeError, exc:
-            raise HTTP400('unable to decode tiddler, utf-8 expected: %s', exc)
+        _process_request_body(environ, tiddler)
 
     try:
         recipe_name = web.get_route_value(environ, 'recipe_name')
@@ -225,6 +225,37 @@ def _post_tiddler_revisions(environ, start_response, tiddler):
     return []
 
 
+def _process_request_body(environ, tiddler):
+    """
+    Read request body to set tiddler.text.
+    """
+    length, content_type = web.content_length_and_type(environ)
+    content = web.read_request_body(environ, length)
+
+    try:
+        try:
+            # XXX HACK! We don't want to decode content unless
+            # the serializer has a as_tiddler. We should be able
+            # to just rely on NoSerializationError, but we need
+            # to call the method to do that, and to call the method we
+            # need to decode the string...
+            serialize_type = web.get_serialize_type(environ)[0]
+            serializer = Serializer(serialize_type, environ)
+            serializer.object = tiddler
+            try:
+                serializer.from_string(content.decode('utf-8'))
+            except TiddlerFormatError, exc:
+                raise HTTP400('unable to put tiddler: %s' % exc)
+        except NoSerializationError:
+            tiddler.type = content_type
+            if pseudo_binary(tiddler.type):
+                tiddler.text = content.decode('utf-8')
+            else:
+                tiddler.text = content
+    except UnicodeDecodeError, exc:
+        raise HTTP400('unable to decode tiddler, utf-8 expected: %s', exc)
+
+
 def _store_tiddler_revisions(environ, content, tiddler):
     """
     Given json revisions in content, store them
@@ -306,7 +337,7 @@ def _put_tiddler(environ, start_response, tiddler):
 
         store.put(tiddler)
     except NoBagError, exc:
-        raise HTTP409("Unable to put tiddler, %s. There is no bag named: " \
+        raise HTTP409("Unable to put tiddler, %s. There is no bag named: "
                 "%s (%s). Create the bag." %
                 (tiddler.title, tiddler.bag, exc))
     except NoTiddlerError, exc:
@@ -377,12 +408,7 @@ def _validate_tiddler_headers(environ, tiddler):
             last_modified_string = web.http_date_from_timestamp(
                     tiddler.modified)
             last_modified = ('Last-Modified', last_modified_string)
-            incoming_modified = environ.get('HTTP_IF_MODIFIED_SINCE', None)
-            if incoming_modified:
-                incoming_modified = web.datetime_from_http_date(incoming_modified)
-                if incoming_modified and (incoming_modified >=
-                        web.datetime_from_http_date(last_modified_string)):
-                    raise HTTP304('')
+            web.check_last_modified(environ, last_modified_string)
 
     else:
         incoming_etag = environ.get('HTTP_IF_MATCH', None)
@@ -446,7 +472,7 @@ def _send_tiddler(environ, start_response, tiddler):
 
     # Add charset to pseudo_binary tiddler
     if not serialized and pseudo_binary(tiddler.type):
-            mime_type = '%s; charset=UTF-8' % mime_type
+        mime_type = '%s; charset=UTF-8' % mime_type
     content_header = ('Content-Type', str(mime_type))
 
     response = [cache_header, content_header, vary_header]
